﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;

namespace PhysXTanks.Input
{
    class InputManager
    {
        private static readonly InputManager instance = new InputManager();

        private InputManager()
        {
            keyboardListeners = new HashSet<KeyboardListener>();
            mouseListeners = new HashSet<MouseListener>();

            previousKeyboardState = new KeyboardState();
            previousMouseState = new MouseState();
        }

        public static InputManager Instace
        {
            get
            {
                return instance;
            }
        }

        //Mange Listeners
        #region Keyboard Listeners Add/Remove

        public void AddKeyboardListener(KeyboardListener listener)
        {
            this.keyboardListeners.Add(listener);
        }
        public bool RemoveKeyboardListener(KeyboardListener listener)
        {
            return this.keyboardListeners.Remove(listener);
        }
        //Previous keyboard state update
        public void UpdatePreviousKeyBoardState(KeyboardState state)
        {
            this.previousKeyboardState = state;
        }

        #endregion


        #region Mouse Listeners Add/Remove

        public void AddMouseListener(MouseListener listener)
        {
            this.mouseListeners.Add(listener);
        }
        public bool RemoveMouseListener(MouseListener listener)
        {
            return this.mouseListeners.Remove(listener);
        }

        #endregion

        HashSet<KeyboardListener> keyboardListeners;
        HashSet<MouseListener> mouseListeners;

        public void Update()
        {
            KeyboardState currentKeyboardStates = Keyboard.GetState();
            Keys[] pressedKeys = currentKeyboardStates.GetPressedKeys();
            MouseState currentMouseState = Mouse.GetState();

            switch (CurrentState.Instance.GameState)
            {
                case GameState.Running:
                    foreach (Keys key in pressedKeys)
                    {
                        if (previousKeyboardState.IsKeyUp(key))
                        {
                            NotifyKeyPressedRunning(key);
                        }
                        else if (previousKeyboardState.IsKeyDown(key))
                        {
                            NotifyKeyHoldedRunning(key);
                        }
                    }
                    //Left mouse button pressed
                    if (previousMouseState.LeftButton == ButtonState.Released && currentMouseState.LeftButton == ButtonState.Pressed)
                    {
                        NotifyMouseLeftButtonPressed();
                    }
                    //Right mouse button pressed
                    if (previousMouseState.RightButton == ButtonState.Released && currentMouseState.RightButton == ButtonState.Pressed)
                    {
                        NotifyMouseRightButtonPressed();
                    }
                    break;
                case GameState.InMenu:
                    foreach (Keys key in pressedKeys)
                    {
                        if (previousKeyboardState.IsKeyUp(key))
                        {
                            NotifyKeyPressedInMenu(key);
                        }
                    }
                    break;
                default:
                    break;
            }
            //Mouse moved
            if ((currentMouseState.X != previousMouseState.X) || (currentMouseState.Y != previousMouseState.Y))
            {
                NotifyMouseMoved(currentMouseState);
            }
            //key pressed
            foreach (Keys key in pressedKeys)
            {
                if (previousKeyboardState.IsKeyUp(key))
                {
                    NotifyKeyPressed(key);
                }
            }
            previousKeyboardState = currentKeyboardStates;
            previousMouseState = currentMouseState;
        }

        #region Keyboard methods


        private void NotifyKeyPressed(Keys key)
        {
            foreach (KeyboardListener listener in keyboardListeners)
            {
                listener.KeyPressed(key);
            }
        }

        private void NotifyKeyPressedRunning(Keys key)
        {
            foreach (KeyboardListener listener in keyboardListeners)
            {
                listener.KeyPressedRunning(key);
            }
        }
        private void NotifyKeyHoldedRunning(Keys key)
        {
            foreach (KeyboardListener listener in keyboardListeners)
            {
                listener.KeyHoldedRunning(key);
            }
        }
        private void NotifyKeyPressedInMenu(Keys key)
        {
            foreach (KeyboardListener listener in keyboardListeners)
            {
                listener.KeyPressedInMenu(key);
            }
        }

        #endregion

        #region Mouse methods

        private void NotifyMouseLeftButtonPressed()
        {
            foreach (MouseListener listener in mouseListeners)
            {
                listener.MouseLeftButtonPressed();
            }
        }
        private void NotifyMouseRightButtonPressed()
        {
            foreach (MouseListener listener in mouseListeners)
            {
                listener.MouseRightButtonPressed();
            }
        }
        private void NotifyMouseMoved(MouseState mouseState)
        {
            foreach (MouseListener listener in mouseListeners)
            {
                listener.MouseMoved(mouseState);
            }
        }

        #endregion

        private KeyboardState previousKeyboardState;
        private MouseState previousMouseState;
    }
}