﻿using System;
using System.Threading;
using Microsoft.Xna.Framework;

using Lidgren.Network;

namespace XnaGameServer
{
    class Server
    {
        static void Main(string[] args)
        {
            NetPeerConfiguration config = new NetPeerConfiguration("xnaapp");
            config.EnableMessageType(NetIncomingMessageType.DiscoveryRequest);
            config.Port = 14242;

            // create and start server
            NetServer server = new NetServer(config);
            server.Start();

            // schedule initial sending of position updates
            double nextSendUpdates = NetTime.Now;

            //tank position
            Matrix pose = new Matrix();

            //bullets count
            int bulletCount = -1;
            Matrix bulletPose = new Matrix();

            // run until escape is pressed
            while (!Console.KeyAvailable || Console.ReadKey().Key != ConsoleKey.Escape)
            {                
                NetIncomingMessage imsg;
                while ((imsg = server.ReadMessage()) != null)
                {                    
                    switch (imsg.MessageType)
                    {
                        case NetIncomingMessageType.DiscoveryRequest:
                            //
                            // Server received a discovery request from a client; send a discovery response (with no extra data attached)
                            //       
                            NetOutgoingMessage omsg = server.CreateMessage();
                            //first player connected
                            if (server.Connections.Length == 0)
                                omsg.Write("firstPlayer");
                            //second player connected
                            else if (server.Connections.Length == 1)
                                omsg.Write("secondPlayer");
                            server.SendDiscoveryResponse(omsg, imsg.SenderEndpoint);
                            break;
                        case NetIncomingMessageType.VerboseDebugMessage:
                        case NetIncomingMessageType.DebugMessage:
                        case NetIncomingMessageType.WarningMessage:
                        case NetIncomingMessageType.ErrorMessage:
                            //
                            // Just print diagnostic messages to console
                            //
                            Console.WriteLine(imsg.ReadString());
                            break;
                        case NetIncomingMessageType.StatusChanged:
                            NetConnectionStatus status = (NetConnectionStatus)imsg.ReadByte();
                            if (status == NetConnectionStatus.Connected)
                            {
                                //
                                // A new player just connected!
                                //                                
                                Console.WriteLine(NetUtility.ToHexString(imsg.SenderConnection.RemoteUniqueIdentifier) + " connected!");                                                               
                            }
                            break;
                        case NetIncomingMessageType.Data:
                            //
                            // The client sent input to the server
                            //               
                            Int32 messageID = imsg.ReadInt32();
                            ReadMatrix(imsg, ref pose);
                            bulletCount = imsg.ReadInt32();
                            ReadMatrix(imsg, ref bulletPose);
                                                                                 
                            float[] pos = imsg.SenderConnection.Tag as float[];

                            pos[0] = messageID;

                            //copy tank GlobalPose to pos[] array
                            pos[1] = pose.M11;
                            pos[2] = pose.M12;
                            pos[3] = pose.M13;
                            pos[4] = pose.M14;
                            pos[5] = pose.M21;
                            pos[6] = pose.M22;
                            pos[7] = pose.M23;
                            pos[8] = pose.M24;
                            pos[9] = pose.M31;
                            pos[10] = pose.M32;
                            pos[11] = pose.M33;
                            pos[12] = pose.M34;
                            pos[13] = pose.M41;
                            pos[14] = pose.M42;
                            pos[15] = pose.M43;
                            pos[16] = pose.M44;

                            pos[17] = bulletCount;

                            //copy bullet GlobalPose to pos[] array
                            pos[18] = bulletPose.M11;
                            pos[19] = bulletPose.M12;
                            pos[20] = bulletPose.M13;
                            pos[21] = bulletPose.M14;
                            pos[22] = bulletPose.M21;
                            pos[23] = bulletPose.M22;
                            pos[24] = bulletPose.M23;
                            pos[25] = bulletPose.M24;
                            pos[26] = bulletPose.M31;
                            pos[27] = bulletPose.M32;
                            pos[28] = bulletPose.M33;
                            pos[29] = bulletPose.M34;
                            pos[30] = bulletPose.M41;
                            pos[31] = bulletPose.M42;
                            pos[32] = bulletPose.M43;
                            pos[33] = bulletPose.M44;
                            break;
                    }
                    //server.Recycle(imsg);

                    //
                    // send position updates same as in Game1.Update() 
                    //
                    double now = NetTime.Now;
                    if (now >= nextSendUpdates)
                    {
                        // send possition updates for each player
                        foreach (NetConnection player in server.Connections)
                        {
                            // ... send information about every other player (actually including self)
                            foreach (NetConnection otherPlayer in server.Connections)
                            {
                                // send position update about 'otherPlayer' to 'player'
                                NetOutgoingMessage omsg = server.CreateMessage();

                                // write who this position is for
                                omsg.Write(otherPlayer.RemoteUniqueIdentifier);

                                if (otherPlayer.Tag == null)
                                    otherPlayer.Tag = new float[34];

                                float[] pos = otherPlayer.Tag as float[];

                                omsg.Write(pos[0]);

                                //tank pose
                                WriteMatrix(omsg, new Matrix(pos[1], pos[2], pos[3], pos[4],
                                                             pos[5], pos[6], pos[7], pos[8],
                                                             pos[9], pos[10], pos[11], pos[12],
                                                             pos[13], pos[14], pos[15], pos[16]));
                                //number of bullets
                                omsg.Write(pos[17]);
                                
                                //bullet pose
                                WriteMatrix(omsg, new Matrix(pos[18], pos[19], pos[20], pos[21],
                                                             pos[22], pos[23], pos[24], pos[25],
                                                             pos[26], pos[27], pos[28], pos[29],
                                                             pos[30], pos[31], pos[32], pos[33]));                                

                                server.SendMessage(omsg, player, NetDeliveryMethod.ReliableOrdered);
                            }
                        }
                        // schedule next update
                        nextSendUpdates += 0.0166667; //same as in Update()                                               
                    }
                }
                // sleep to allow other processes to run smoothly
                //Thread.Sleep(1);
            }
            server.Shutdown("app exiting");
        }

        public static void WriteVector3(NetOutgoingMessage omsg, Vector3 vector)
        {
            omsg.Write(vector.X);
            omsg.Write(vector.Y);
            omsg.Write(vector.Z);
        }

        /// <summary>
        /// Reads a Vector3
        /// </summary>
        public static Vector3 ReadVector3(NetIncomingMessage imsg)
        {
            Vector3 retval;
            retval.X = imsg.ReadSingle();
            retval.Y = imsg.ReadSingle();
            retval.Z = imsg.ReadSingle();
            return retval;
        }

        /// <summary>
        /// Writes an orthonormal matrix (rotation, translation but no scaling or projection)
        /// </summary>
        public static void WriteMatrix(NetOutgoingMessage omsg, Matrix matrix)
        {
            omsg.Write(matrix.M11);
            omsg.Write(matrix.M12);
            omsg.Write(matrix.M13);
            omsg.Write(matrix.M14);
            omsg.Write(matrix.M21);
            omsg.Write(matrix.M22);
            omsg.Write(matrix.M23);
            omsg.Write(matrix.M24);
            omsg.Write(matrix.M31);
            omsg.Write(matrix.M32);
            omsg.Write(matrix.M33);
            omsg.Write(matrix.M34);
            omsg.Write(matrix.M41);
            omsg.Write(matrix.M42);
            omsg.Write(matrix.M43);
            omsg.Write(matrix.M44);
        }

        /// <summary>
        /// Reads a matrix written using WriteMatrix()
        /// </summary>
        public static void ReadMatrix(NetIncomingMessage imsg, ref Matrix destination)
        {
            destination.M11 = imsg.ReadSingle();
            destination.M12 = imsg.ReadSingle();
            destination.M13 = imsg.ReadSingle();
            destination.M14 = imsg.ReadSingle();
            destination.M21 = imsg.ReadSingle();
            destination.M22 = imsg.ReadSingle();
            destination.M23 = imsg.ReadSingle();
            destination.M24 = imsg.ReadSingle();
            destination.M31 = imsg.ReadSingle();
            destination.M32 = imsg.ReadSingle();
            destination.M33 = imsg.ReadSingle();
            destination.M34 = imsg.ReadSingle();
            destination.M41 = imsg.ReadSingle();
            destination.M42 = imsg.ReadSingle();
            destination.M43 = imsg.ReadSingle();
            destination.M44 = imsg.ReadSingle();         
        }
    }
}
